/****************************************************************************
 *                                                                          
 * Title:   Error Handling & Time Profiling
 *
 * Purpose: This example shows basic principles of error handling.
 *
 ****************************************************************************/

#include <ansi_c.h>
#include "rssifs.h"

/*** Macros & definitions ***************************************************/

#define CHECKERR( fCal ) \
    { \
        fCalStartTime = clock();\
        error = (fCal);\
        status = error;\
        printf(" Line %ld (%ld ms): %s\n", __LINE__,\
            (fCalTime = (clock() - fCalStartTime)), #fCal);\
        if (error != VI_SUCCESS)\
            {\
            rssifs_error_message (io, error, error_message);\
            printf("\tFunction Call Status: 0x%08X, %s\n", error, error_message);\
            rssifs_error_query(io, &error, error_message);\
            printf("\tInstrument Error: 0x%X, %s\n", error, error_message);\
            }\
    }

#define RESOURCE_NAME   "USB::0xAAD::0x6::100202"   // Resource name

/*** Main *******************************************************************/

int main (int argc, char *argv[])
{
    ViStatus    error               = VI_SUCCESS,
                status              = VI_SUCCESS;
    clock_t     fCalTime            = 0,
                fCalStartTime       = 0;
    ViChar      error_message[256];
    ViSession   io;
    
    ViReal64    sweep_time          = 0.0;
    
    /* Define remote connection parameters (as default values) */

    ViBoolean   IDQuery             = VI_TRUE,
                resetDevice         = VI_TRUE;
    ViRsrc      resourceName        = RESOURCE_NAME;

    CHECKERR (rssifs_init (resourceName, IDQuery, resetDevice, &io));

    printf (

    "\n --- Error Checking ---\n\n"
    "\tThis example uses macro CHECKERR to cover error handling.\n"
    "\tBasic principle of error handling is as follows:\n\n"

    "\t(1) status code (status) is returned by function call (fCal)\n"
    "\t(2) if status code is <> VI_SUCCESS (0):\n"
    "\t    - translate status code to message using rssifs_error_message"
    " function\n"
    "\t    - call rssifs_error_query function to get more information from error"
    " queue\n"
    "\t(3) if status code is equal to VI_SUCCESS (0):\n"
    "\t    - function call succeed\n\n"

    );

    /* Correct function call */
    CHECKERR (rssifs_confStartStopFrq (io, 0, 0.0, 3.0e9));
    /* Passed wrong data to generate error */
    CHECKERR (rssifs_confStartStopFrq (io, 0, 3.0e9, 0.0));

    printf (

    "\n --- Warning Checking ---\n\n"
    "\tWarning checking provides additional information of device operations.\n"
    "\tWarnings are produced by the instrument as asynchronous messages.\n"
    "\tIt can be enabled using function rssifs_warningCheckState.\n"
    "\tHandling of the warnings is the same as for errors.\n\n"

    );

    CHECKERR (rssifs_warningCheckState (io, VI_TRUE));
    /* Try to produce warnings ... */
    CHECKERR (rssifs_confSweep (io, 1, 0));
    CHECKERR (rssifs_confSpanCenterFrq (io, 0, 0.0, 1.0e9));
    CHECKERR (rssifs_getSweepTime (io, &sweep_time));
    CHECKERR (rssifs_warningCheckState (io, VI_FALSE));

    printf (

    "\n --- Disable Warning & Error Checking ---\n\n"
    "\tWarning & Error checking can be disabled by calling function"
    " rssifs_errorCheckState.\n"
    "\tSee description:\n\n"

    "\t(1) When disabled, status checking is not performed and function\n"
    "\t    rssifs_error_query do not provide any error message\n"
    "\t    information.\n"

    "\t(2) When disabled, internal event handling mechanism (interrupt\n"
    "\t    pipe checking & control transfer handshake checking) is also\n"
    "\t    disabled.\n"

    "\t(3) When disabled, interaction in between instrument driver on\n"
    "\t    the host computer and device's firmware is affected. Performance\n"
    "\t    of the instrument driver calls might increase, but behavior is\n"
    "\t    not fully predictable (synchronization and timing mechanism is\n"
    "\t    disabled).\n\n"

    );

    printf (

    "\n --- Performance Improvement ---\n\n"

    "\tWhen the error checking is disabled, performance might increase:\n\n"
    
    );

    CHECKERR (rssifs_confStartStopFrq (io, 0, 0.0, 3.0e9));
    CHECKERR (rssifs_confStartStopFrq (io, 0, 0.0, 3.0e9));
    CHECKERR (rssifs_errorCheckState (io, VI_FALSE));
    CHECKERR (rssifs_confStartStopFrq (io, 0, 0.0, 3.0e9));
    CHECKERR (rssifs_confStartStopFrq (io, 0, 0.0, 3.0e9));

    printf (

    "\n\tWARNING: Use rssifs_errorCheckState function with care!\n\n"
    
    );

    CHECKERR (rssifs_close (io));

    return 0;
}
