/****************************************************************************
 *                                                                          
 * Title:   Marker(s) Measurement
 *
 * Purpose: This example shows how to proceed with marker measurement.
 *
 ****************************************************************************/

#include <ansi_c.h>
#include "rssifs.h"

/*** Macros & definitions ***************************************************/

#define CHECKERR( fCal ) \
    { \
        fCalStartTime = clock();\
        error = (fCal);\
        status = error;\
        printf(" Line %ld (%ld ms): %s\n", __LINE__,\
            (fCalTime = (clock() - fCalStartTime)), #fCal);\
        if (error != VI_SUCCESS)\
            {\
            rssifs_error_message (io, error, error_message);\
            printf("\tFunction Call Status: 0x%08X, %s\n", error, error_message);\
            rssifs_error_query(io, &error, error_message);\
            printf("\tInstrument Error: 0x%X, %s\n", error, error_message);\
            }\
    }

#define RESOURCE_NAME   "USB::0xAAD::0x6::100202"   // Resource name

/*** Main *******************************************************************/

int main (int argc, char *argv[])
{
    ViStatus    error               = VI_SUCCESS,
                status              = VI_SUCCESS;
    clock_t     fCalTime            = 0,
                fCalStartTime       = 0;
    ViChar      error_message[256];
    ViSession   io;
    
    /* Define remote connection parameters (as default values) */

    ViBoolean   IDQuery             = VI_TRUE,
                resetDevice         = VI_TRUE;
    ViRsrc      resourceName        = RESOURCE_NAME;

    ViReal64    marker_position     = 0.0,
                marker_value        = 0.0;

    CHECKERR (rssifs_init (resourceName, IDQuery, resetDevice, &io));

    printf (

    "\n --- Marker(s) Measurement ----------------------------------------------\n\n"
    "\tRF Generator is connected to the analyzer's input\n"
    "\t- Frequency: 1 GHz\n"
    "\t- Level: -30.0 dBm\n"
    "\t- Modulation: AM, modulation frequency 50 kHz, modulation depth 100 %%\n"
    "\n ------------------------------------------------------------------------\n\n"

    );

    printf ("\n --- Set center 1 GHz and span 0.5 MHz\n\n");
    CHECKERR (rssifs_confSpanCenterFrq (io, 0, 500.0e3, 1.0e9));

    /* For higher resolution is set 500 measured points per sweep */
    CHECKERR (rssifs_confSweepPoints (io, 500));
    /* Configure RBW & VBW */
    CHECKERR (rssifs_configureBandwidth (io, 8, 15));
    /* Set reference level */
    CHECKERR (rssifs_confRefLevel (io, -20.0));
    /* Set single sweep mode */
    CHECKERR (rssifs_confSweep (io, 1, 0));

    printf ("\n --- Send trigger and wait for operation completed\n\n");
    CHECKERR (rssifs_actSendTrgWopc (io, 20000));

    /* Enable markers */
    CHECKERR (rssifs_confMarkState (io, 1, VI_TRUE));
    CHECKERR (rssifs_confMarkState (io, 2, VI_TRUE));
    
    printf ("\n --- Search for max peak (marker 1 and 2)\n\n");

    CHECKERR (rssifs_actMarkSearch (io, 1, 0));
    CHECKERR (rssifs_actMarkSearch (io, 2, 0));

    CHECKERR (rssifs_getMarkerPosition (io, 1, &marker_position));
    CHECKERR (rssifs_readMarkerValue (io, 1, &marker_value));

    printf ("\n\t1st Marker position [Hz]: %.1Lf\n", marker_position);
    printf ("\t1st Marker Value [Vrms]: %Le\n", marker_value);

    CHECKERR (rssifs_getMarkerPosition (io, 2, &marker_position));
    CHECKERR (rssifs_readMarkerValue (io, 2, &marker_value));
        
    printf ("\n\t2nd Marker position [Hz]: %.1Lf\n", marker_position);
    printf ("\t2nd Marker Value [Vrms]: %Le\n", marker_value);

    printf ("\n --- Search for the left maximum (marker 1, absolute search mode)\n\n");

    CHECKERR (rssifs_actMarkSearch (io, 1, 2));
    
    CHECKERR (rssifs_getMarkerPosition (io, 1, &marker_position));
    CHECKERR (rssifs_readMarkerValue (io, 1, &marker_value));
        
    printf ("\n\t1st Marker position [Hz]: %.1Lf\n", marker_position);
    printf ("\t1st Marker Value [Vrms]: %Le\n", marker_value);

    printf ("\n --- Search for the right maximum (marker 2, absolute search mode)\n\n");

    CHECKERR (rssifs_actMarkSearch (io, 2, 3));
    
    CHECKERR (rssifs_getMarkerPosition (io, 2, &marker_position));
    CHECKERR (rssifs_readMarkerValue (io, 2, &marker_value));
        
    printf ("\n\t2nd Marker position [Hz]: %.1Lf\n", marker_position);
    printf ("\t2nd Marker Value [Vrms]: %Le\n", marker_value);

    CHECKERR (rssifs_close (io));

    return 0;
}
