/****************************************************************************
 *                                                                          
 * Title:   Sweep Time & Resolution Bandwidth & Video Bandwidth
 *
 * Purpose: This example shows dependencies of sweep time, resolution
 *          bandwidth and video bandwidth.
 *
 ****************************************************************************/

#include <ansi_c.h>
#include "rssifs.h"

/*** Macros & definitions ***************************************************/

#define CHECKERR( fCal ) \
    { \
        fCalStartTime = clock();\
        error = (fCal);\
        status = error;\
        printf(" Line %ld (%ld ms): %s\n", __LINE__,\
            (fCalTime = (clock() - fCalStartTime)), #fCal);\
        if (error != VI_SUCCESS)\
            {\
            rssifs_error_message (io, error, error_message);\
            printf("\tFunction Call Status: 0x%08X, %s\n", error, error_message);\
            rssifs_error_query(io, &error, error_message);\
            printf("\tInstrument Error: 0x%X, %s\n", error, error_message);\
            }\
    }

#define RESOURCE_NAME   "USB::0xAAD::0x6::100202"   // Resource name

/*** Main *******************************************************************/

int main (int argc, char *argv[])
{
    ViStatus    error               = VI_SUCCESS,
                status              = VI_SUCCESS;
    clock_t     fCalTime            = 0,
                fCalStartTime       = 0;
    ViChar      error_message[256];
    ViSession   io;
    
    /* Define remote connection parameters (as default values) */

    ViBoolean   IDQuery             = VI_TRUE,
                resetDevice         = VI_TRUE;
    ViRsrc      resourceName        = RESOURCE_NAME;

    ViReal64    sweep_time          = 0.0,
                RBW                 = 0.0,
                VBW                 = 0.0;

    CHECKERR (rssifs_init (resourceName, IDQuery, resetDevice, &io));

    printf (

    "\n --- Sweep Time & Resolution Bandwidth & Video Bandwidth ----------------\n\n"
    "\tRF Generator is connected to the analyzer's input\n"
    "\t- Frequency: 1 GHz\n"
    "\t- Level: -30.0 dBm\n"
    "\t- Modulation: AM, modulation frequency 50 kHz, modulation depth 100 %%\n"
    "\n ------------------------------------------------------------------------\n\n"

    );

    /* Set reference level */
    CHECKERR (rssifs_confRefLevel (io, -20.0));
    /* Set single sweep mode */
    CHECKERR (rssifs_confSweep (io, 1, 0));

    printf ("\n --- Auto adjusting of RBW, VBW and Sweep Time\n\n");

    CHECKERR (rssifs_confSweepTime (io, 0.0));
    CHECKERR (rssifs_configureBandwidth (io, 0, 0));

    printf ("\n --- Set center 1 GHz and span 500 kHz\n\n");

    CHECKERR (rssifs_confSpanCenterFrq (io, 0, 500.0e3, 1.0e9));
    CHECKERR (rssifs_actSendTrg (io));

    printf ("\nGet settings:\n\n");

    CHECKERR (rssifs_getSweepTime (io, &sweep_time));
    CHECKERR (rssifs_getResolutionBandwidth (io, &RBW));
    CHECKERR (rssifs_getVideoBandwidth (io, &VBW));

    printf ("\n\tSweep time [s]: %.3Lf\n", sweep_time);
    printf ("\tRBW [Hz]: %.3Lf\n", RBW);
    printf ("\tVBW [Hz]: %.3Lf\n", VBW);

    printf ("\n --- Set RBW to 100 kHz, VBW is auto\n\n");

    CHECKERR (rssifs_confResBW (io, 100.0e3));
    CHECKERR (rssifs_actSendTrg (io));

    printf ("\nGet settings:\n\n");

    CHECKERR (rssifs_getSweepTime (io, &sweep_time));
    CHECKERR (rssifs_getResolutionBandwidth (io, &RBW));
    CHECKERR (rssifs_getVideoBandwidth (io, &VBW));

    printf ("\n\tSweep time [s]: %.3Lf\n", sweep_time);
    printf ("\tRBW [Hz]: %.3Lf\n", RBW);
    printf ("\tVBW [Hz]: %.3Lf\n\n", VBW);

    CHECKERR (rssifs_close (io));

    return 0;
}
