/****************************************************************************
 *                                                                          
 * Title:   Measurement in Time Domain
 *
 * Purpose: This example shows how to setup measurement in time domain
 *          (Zero span measurement).
 *
 ****************************************************************************/

#include <ansi_c.h>
#include "rssifs.h"

/*** Macros & definitions ***************************************************/

#define CHECKERR( fCal ) \
    { \
        fCalStartTime = clock();\
        error = (fCal);\
        status = error;\
        printf(" Line %ld (%ld ms): %s\n", __LINE__,\
            (fCalTime = (clock() - fCalStartTime)), #fCal);\
        if (error != VI_SUCCESS)\
            {\
            rssifs_error_message (io, error, error_message);\
            printf("\tFunction Call Status: 0x%08X, %s\n", error, error_message);\
            rssifs_error_query(io, &error, error_message);\
            printf("\tInstrument Error: 0x%X, %s\n", error, error_message);\
            }\
    }

#define RESOURCE_NAME   "USB::0xAAD::0x6::100202"   // Resource name

/*** Main *******************************************************************/

int main (int argc, char *argv[])
{
    ViStatus    error               = VI_SUCCESS,
                status              = VI_SUCCESS;
    clock_t     fCalTime            = 0,
                fCalStartTime       = 0;
    ViChar      error_message[256];
    ViSession   io;
    
    /* Define remote connection parameters (as default values) */

    ViBoolean   IDQuery             = VI_TRUE,
                resetDevice         = VI_TRUE;
    ViRsrc      resourceName        = RESOURCE_NAME;

    CHECKERR (rssifs_init (resourceName, IDQuery, resetDevice, &io));

    printf (

    "\n --- Measurement in Time Domain ----------------------------------------\n\n"
    "\tRF Generator is connected to the analyzer's input\n"
    "\t- Frequency: 1 GHz\n"
    "\t- Level: -30.0 dBm\n"
    "\t- Modulation: AM, modulation frequency 1 kHz, modulation depth 100 %%\n"
    "\n ------------------------------------------------------------------------\n\n"

    );

    /* Set reference level */
    CHECKERR (rssifs_confRefLevel (io, -20.0));
    /* Set center frequency and span */
    CHECKERR (rssifs_confSpanCenterFrq (io, 0, 0.0, 1.0e9));
    /* Set RBW = 1MHz and VBW = 1MHz */
    CHECKERR (rssifs_configureBandwidth (io, 16, 21));
    /* Configure Sweep Time to 2 ms */
    CHECKERR (rssifs_confSweepTime (io, 0.002));
    
    /* Read trace data of completed sweep */
    {
        ViInt32     samplesReturned = 0;
        ViReal64    sweepTime       = 0.0,
                    traceData[2048];

        CHECKERR (rssifs_readCompleteSweepData (io, traceData, &samplesReturned));

        if (error == VI_SUCCESS)
            {
            CHECKERR (rssifs_getSweepTime (io, &sweepTime));

            printf ("\tSamples Returned: %ld\n"
                    "\tTrace Data [Vrms]: %.3Le, %.3Le, %.3Le ...\n"
                    "\tStart Time [s]: %Lf\n"
                    "\tStop Time [s]: %Lf\n"
                    "\tTime Step [s]: %.3Le\n",
                    samplesReturned,
                    traceData[0], traceData[1], traceData[2],
                    0.0,
                    sweepTime,
                    (sweepTime / samplesReturned));
            }
    }
    
    CHECKERR (rssifs_close (io));

    return 0;
}
