/****************************************************************************
 *                                                                          
 * Title:   Evaluation of Trace Data
 *
 * Purpose: This example shows how to handle trace data.
 *
 ****************************************************************************/

#include <ansi_c.h>
#include "rssifs.h"

/*** Macros & definitions ***************************************************/

#define CHECKERR( fCal ) \
    { \
        fCalStartTime = clock();\
        error = (fCal);\
        status = error;\
        printf(" Line %ld (%ld ms): %s\n", __LINE__,\
            (fCalTime = (clock() - fCalStartTime)), #fCal);\
        if (error != VI_SUCCESS)\
            {\
            rssifs_error_message (io, error, error_message);\
            printf("\tFunction Call Status: 0x%08X, %s\n", error, error_message);\
            rssifs_error_query(io, &error, error_message);\
            printf("\tInstrument Error: 0x%X, %s\n", error, error_message);\
            }\
    }

#define RESOURCE_NAME   "USB::0xAAD::0x6::100202"   // Resource name

/*** Main *******************************************************************/

int main (int argc, char *argv[])
{
    ViStatus    error               = VI_SUCCESS,
                status              = VI_SUCCESS;
    clock_t     fCalTime            = 0,
                fCalStartTime       = 0;
    ViChar      error_message[256];
    ViSession   io;
    
    ViReal64    sweep_time          = 0.0,
                traceData[2048];
                
    ViInt32     samplesReturned     = 0;

    /* Define remote connection parameters (as default values) */

    ViBoolean   IDQuery             = VI_TRUE,
                resetDevice         = VI_TRUE;
    ViRsrc      resourceName        = RESOURCE_NAME;

    CHECKERR (rssifs_init (resourceName, IDQuery, resetDevice, &io));

    printf (

    "\n --- Evaluation of Trace Data -------------------------------------------\n\n"
    "\tRF Generator is connected to the analyzer's input\n"
    "\t- Frequency: 1 GHz\n"
    "\t- Level: -30.0 dBm\n"
    "\t- Modulation: AM, modulation frequency 5 kHz, modulation depth 100 %%\n"
    "\n ------------------------------------------------------------------------\n\n"

    );

    /* Set reference level */
    CHECKERR (rssifs_confRefLevel (io, -20.0));
    /* Set RBW to 200 kHz VBW to 500 kHz */
    CHECKERR (rssifs_configureBandwidth (io, 13, 20));
    /* Set start and stop frequency */
    CHECKERR (rssifs_confStartStopFrq (io, 0, 999.0e6, 001e9));
    /* Get Sweep Time */
    CHECKERR (rssifs_getSweepTime (io, &sweep_time));
    printf ("\tSweep time is adjusted to %.3Lf s\n", sweep_time);

    printf ("\n --- Read complete sweep (Clear/Write mode)\n\n");

    /* Configure Trace Mode Clear/Write */
    CHECKERR (rssifs_confTraceMode (io, 0));
    /* Read trace data */
    CHECKERR (rssifs_readCompleteSweepData (io, traceData, &samplesReturned));

    printf ("\n --- Read complete sweep (Average mode)\n\n");

    /* Configure single sweep mode to 10 sweeps per trigger */
    CHECKERR (rssifs_confSweep (io, 2, 10));
    /* Configure Trace Mode to Average */
    CHECKERR (rssifs_confTraceMode (io, 2));
    /* Start measurement and read the trace data */
    CHECKERR (rssifs_actSendTrgWopc (io, 5000));
    CHECKERR (rssifs_readCompleteSweepData (io, traceData, &samplesReturned));

    printf (
    
    "\n\tWarning code informs you that sweep data are after trigger\n"
    "\tis executed stored to the driver cache and function\n"
    "\trssifs_readCompleteSweepData is not reading data from instrument.\n"
    
    );

    printf ("\n --- Read raw trace data (bypassing trace caching mechanism)\n\n");

    {
        ViReal64    data[2048],
                    tempData[2048];
        ViInt32     startIndex      = 0,
                    samplesCount    = 0,
                    sweepPoints     = 500;

        /* Configure start and stop frequency */
        CHECKERR( rssifs_confStartStopFrq (io, 0, 500.0e6, 1.5e9));
        /* Set RBW to 100 kHz VBW to 300 kHz */
        CHECKERR (rssifs_configureBandwidth (io, 12, 19));
        /* Configure number of sweep points */
        CHECKERR( rssifs_confSweepPoints (io, sweepPoints));
        /* Configure continuous sweep mode */
        CHECKERR( rssifs_confSweep (io, 0, 0));
        /* Configure Sweep Time */
        CHECKERR (rssifs_confSweepTime (io, 1.0));
        CHECKERR (rssifs_getSweepTime (io, &sweep_time));
        printf ("\tSweep time is adjusted to %.3Lf s\n", sweep_time);

        printf ("\n ... Restart measurement and wait for result\n\n");
        CHECKERR( rssifs_actSendTrgWopc (io, 5000));

        printf ("\n ... Do once dummy read of trace data (flush instrument buffer)\n\n");
        
        CHECKERR( rssifs_readTraceData (io, data, &startIndex, &samplesCount));
        printf ("\tStart index = \"%ld\", samples returned = \"%ld\"\n",
            startIndex, samplesCount);

        /* Delay for cca 1300 millisecond (illustrative example) */
        {
            clock_t start_time = clock();

            while ((long)(clock() - start_time) < 1300);
        }

        /* --- Simplified method of reading raw trace data --- */

        printf ("\n --- Simplified method of reading raw trace data\n\n");
        
        CHECKERR (rssifs_readTraceData (io, data, &startIndex, &samplesCount));
        printf ("\tStart index = \"%ld\", samples returned = \"%ld\"\n\n",
            startIndex, samplesCount);

            {
            ViUInt16    count       = samplesCount,
                        originStart = startIndex;

             while ((count < sweepPoints) && (count < 2048))
                {
                
                CHECKERR (rssifs_readTraceData (io, &tempData[count],
                    &startIndex, &samplesCount));
                printf ("\tStart index = \"%ld\", samples returned = \"%ld\"\n\n",
                    startIndex, samplesCount);

                if (samplesCount == sweepPoints) /* Full sweep? */
                    {
                    originStart = startIndex;
                    break;
                    }
                
                if (samplesCount == 0) /* Empty buffer? */
                    {
                    printf ("\n\t*** Data not available ***\n\n");
                    break;
                    }

                count += samplesCount;
                }

            /* Samples count should be then equal to the number of sweep points */
            samplesCount = sweepPoints;

            /* Place data to the target buffer in proper order */
            memcpy (&traceData[originStart], tempData,
                (sweepPoints - originStart) * sizeof (tempData[0]));
            memcpy (traceData, &tempData[(sweepPoints - originStart)],
                originStart * sizeof (tempData[0]));
            }
    }

    CHECKERR (rssifs_close (io));

    return 0;
}
