/****************************************************************************
 *                                                                          
 * Title:   Triggered Measurement (Single sweep mode)
 *
 * Purpose: This example shows how to setup triggered measurement.
 *
 ****************************************************************************/

#include <ansi_c.h>
#include "rssifs.h"

/*** Macros & definitions ***************************************************/

#define CHECKERR( fCal ) \
    { \
        fCalStartTime = clock();\
        error = (fCal);\
        status = error;\
        printf(" Line %ld (%ld ms): %s\n", __LINE__,\
            (fCalTime = (clock() - fCalStartTime)), #fCal);\
        if (error != VI_SUCCESS)\
            {\
            rssifs_error_message (io, error, error_message);\
            printf("\tFunction Call Status: 0x%08X, %s\n", error, error_message);\
            rssifs_error_query(io, &error, error_message);\
            printf("\tInstrument Error: 0x%X, %s\n", error, error_message);\
            }\
    }

#define RESOURCE_NAME   "USB::0xAAD::0x6::100202"   // Resource name
#define SWEEPS          10                          // Number of sweeps

/*** Main *******************************************************************/

int main (int argc, char *argv[])
{
    ViStatus    error               = VI_SUCCESS,
                status              = VI_SUCCESS;
    clock_t     fCalTime            = 0,
                fCalStartTime       = 0;
    ViChar      error_message[256];
    ViSession   io;
    
    ViReal64    startFrequency      = 500.0e6,
                stopFrequency       = 1.500e9,
                sweep_time          = 1.0,
                traceData[250];
                
    ViInt32     sweep_count         = 0,
                samplesReturned     = 0;

    /* Define remote connection parameters (as default values) */

    ViBoolean   IDQuery             = VI_TRUE,
                resetDevice         = VI_TRUE;
    ViRsrc      resourceName        = RESOURCE_NAME;

    CHECKERR (rssifs_init (resourceName, IDQuery, resetDevice, &io));

    printf (

    "\n --- Triggered Measurement (Single sweep mode) --------------------------\n\n"
    "\tRF Generator is connected to the analyzer's input\n"
    "\t- Frequency: 1 GHz\n"
    "\t- Level: -30.0 dBm\n"
    "\t- Modulation: AM, modulation frequency 1 kHz, modulation depth 100 %%\n"
    "\n ------------------------------------------------------------------------\n\n"

    );

    /* Set reference level */
    CHECKERR (rssifs_confRefLevel (io, -20.0));
    /* Set start and stop frequency */
    CHECKERR (rssifs_confStartStopFrq (io, 0, startFrequency, stopFrequency));
    /* Set RBW to 300 kHz VBW to 1 MHz */
    CHECKERR (rssifs_configureBandwidth (io, 14, 21));

    /* Configure single sweep mode with defined number of sweeps to be executed */
    CHECKERR (rssifs_confSweep (io, 2, SWEEPS));
    /* Configure Sweep Time */
    CHECKERR (rssifs_confSweepTime (io, 1.0));
    CHECKERR (rssifs_getSweepTime (io, &sweep_time));
    printf ("\tSweep time is adjusted to %.3Lf s\n", sweep_time);

/* --- Single Sweep ------------------------------------------------------- */

    printf ("\n --- 1st triggering method: Send Trigger and Wait for operation completed\n\n");
    CHECKERR (rssifs_actSendTrgWopc (io, 20000));
    sweep_count = 0;
    /* Read trace data */
    CHECKERR (rssifs_readCompleteSweepData (io, traceData, &samplesReturned));  

    printf (
    
    "\n\tWarning code informs you that sweep data are after trigger\n"
    "\tis executed stored to the driver cache and function\n"
    "\trssifs_readCompleteSweepData is not reading data from instrument.\n\n"
    
    );

    /* Get number of measured sweeps */
    CHECKERR (rssifs_getSweepCount (io, &sweep_count));
    printf ("\tNumber of measured sweeps: %ld\n", sweep_count);

    printf ("\n --- 2nd triggering method: Send Trigger\n\n");
    CHECKERR (rssifs_actSendTrg (io));
    sweep_count = 0;
    while (sweep_count < SWEEPS)
        {
        /* Read trace data */
        CHECKERR (rssifs_readCompleteSweepData (io, traceData, &samplesReturned));
        /* Get number of measured sweeps */
        CHECKERR (rssifs_getSweepCount (io, &sweep_count));
        printf ("\tNumber of measured sweeps: %ld\n", sweep_count);
        }

/* --- Free Run ----------------------------------------------------------- */

    printf ("\n --- Triggering in the free run mode\n\n");

    /* Zero span mode */
    CHECKERR (rssifs_confSpanCenterFrq (io, 0, 0.0, startFrequency +
        (stopFrequency - startFrequency) / 2));
    /* Configure Sweep Time */
    CHECKERR (rssifs_confSweepTime (io, 0.001));
    CHECKERR (rssifs_getSweepTime (io, &sweep_time));
    printf ("\tSweep time is adjusted to %.3Lf s\n", sweep_time);

    /* Configure free run mode */
    CHECKERR (rssifs_confTrg (io, 0, -60.0, VI_FALSE));

    /* Measurement is (re)started under configured trigger and sweep conditions */
    CHECKERR (rssifs_actSendTrgWopc (io, 5000));
    /* Read trace data */
    CHECKERR (rssifs_readCompleteSweepData (io, traceData, &samplesReturned));

/* --- Video Trigger ------------------------------------------------------ */

    printf ("\n --- Video Trigger (trigger level -40 dBm, trigger offset is zero)\n\n");

    /* Configure Trigger */
    CHECKERR (rssifs_confTrg (io, 1, -40.0, VI_FALSE));
    /* Trigger Offset (trigger delay) */
    CHECKERR (rssifs_confTrgDelay (io, 0.0));

    /* Measurement is (re)started under configured trigger and sweep conditions */
    CHECKERR (rssifs_actSendTrgWopc (io, 5000));
    /* Read trace data */
    printf ("\n\tData are returned only if the trigger condition meets\n\n");
    CHECKERR (rssifs_readCompleteSweepData (io, traceData, &samplesReturned));

    printf ("\n --- Video Trigger (trigger level -40 dBm, offset is -0.2 ms)\n\n");

    /* Configure Trigger */
    CHECKERR (rssifs_confTrg (io, 1, -40.0, VI_FALSE));
    /* Trigger Offset (trigger delay) */
    CHECKERR (rssifs_confTrgDelay (io, -0.2e-3));

    /* Measurement is (re)started under configured trigger and sweep conditions */
    CHECKERR (rssifs_actSendTrgWopc (io, 5000));
    /* Read trace data */
    printf ("\n\tData are returned only if the trigger condition meets\n\n");
    CHECKERR (rssifs_readCompleteSweepData (io, traceData, &samplesReturned));

    printf ("\n");

    CHECKERR (rssifs_close (io));

    return 0;
}
